-------------------------------------------------------------------------------
-- Name: class AttractLoop
-- Author: Jonathan Primeau
--
-- Description:
--     This class takes care of the attract loop
--
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pAttractLoopList      -- The list of attractions to be displayed
property pNextAttraction       -- The next attraction index
property pTotalDuration        -- The length of the attact loop in seconds
property pPlayCount            -- Number of time the attract loop has been played
property pStarted              -- Attract loop running
property pDurationIndexList
property pAttractLoopFolder
property pAttractLoopName

-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class XMLParser").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

on SetAttractLoopName (me, theName)
  me.pAttractLoopName = theName & "\"
end

-------------------------------------------------------------------------------
-- void Reset (me)
--
-- This method loads the attract loop using the XML.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Load (me)
  me.pDurationIndexList = [0]
  
  if (me.pAttractLoopName = "") then
    GenLog(#error, "AttractLoop::Load: Attract loop folder is empty!")
    SvrLog("Attract Loop: Folder not specified")
    return(-1)
  end if
  
  lAttractLoopFolder = "AttractLoops\" & me.pAttractLoopName
  me.pAttractLoopFolder = the moviePath & lAttractLoopFolder
  
  -- Check if the attract loop path exists
  if (QI(#file).FolderExists(me.pAttractLoopFolder) = false) then
    GenLog(#error, "AttractLoop::Load: Attract loop folder doesn't exist" && me.pAttractLoopFolder)
    SvrLog("Attract Loop: Folder doesn't exist" && me.pSkinPath)
    me.pAttractLoopFolder = ""
    return(-2)
  end if
  
  lAttractLoopFile = lAttractLoopFolder & "attract_loop.xml"
  
  -- Load and parse the XML
  me.LoadXMLFromFile(lAttractLoopFile)
  lStatus = me.Parse(#_XMLCallback)
  if (lStatus <> 0) then
    return(lStatus)
  end if
  
  GenLog(#debug, "AttractLoop::Load: Attract loop loaded!")
  return(0)
end

-------------------------------------------------------------------------------
-- void Reset (me)
--
-- This method resets the attract loop.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Reset (me)
  GenLog(#debug, "AttractLoop::Reset: Resetting attract loop...")
  me.StopCycling()
  me.pNextAttraction = 1
  me.pPlayCount = 0
  me.Cycle()
  GenLog(#debug, "AttractLoop::Reset: Attract loop reset!")
end

-------------------------------------------------------------------------------
-- void GetTotalDuration (me)
--
-- This method returns the total attract loop duration.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     An interger which is the total attract loop duration.
--
-------------------------------------------------------------------------------
on GetTotalDuration (me)
  return(me.pTotalDuration)
end

-------------------------------------------------------------------------------
-- void GetStarted (me)
--
-- This method returns true if the attract loop is started.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     True if the attract loop has been started.
--
-------------------------------------------------------------------------------
on GetStarted (me)
  return(me.pStarted) 
end

-------------------------------------------------------------------------------
-- void Clear (me)
--
-- This method clears the attraction sprite and centers it.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Clear (me)
  me._HideSprite()
  if (SpriteExists("attraction")) then
    sprite("attraction").member.filename = the moviePath & "Skins\" & "empty.png"
  end if
  me._CenterSprite()
end

-------------------------------------------------------------------------------
-- void StartCycling (me)
--
-- This method is used to start the timeout for cycling the attract loop.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on StartCycling (me, theTimeout)
  GenLog(#debug, "AttractLoop::StartCycling: Start cycling...")
  
  -- We want to cycle every X miliseconds
  if (theTimeout = void) then
    lTimeout = 5 * 1000 -- 5 seconds * 1000 miliseconds
  else
    lTimeout = theTimeout
  end if
  
  -- Start the banner cycling timeout
  QI(#attractCycleTimeout).SetTimeout(lTimeout)
  QI(#attractCycleTimeout).Start()
end

-------------------------------------------------------------------------------
-- void Cycle (me)
--
-- This method is called by the attract loop cycling timeout callback (global).
-- everytime it needs to cycle the image. It takes care of loading the next
-- attraction image in the cycling loop.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Cycle (me, theIndex)
  me._HideSprite()
  
  if (theIndex <> void) then
    me.pNextAttraction = theIndex
  end if
  
  lCount = me.pAttractLoopList.count
  if (me.pNextAttraction < 1 or me.pNextAttraction > lCount) then
    GenLog(#error, "AttractLoop::Cycle: Invalid attraction index!")
    SvrLog("Attract Loop: Invalid attraction index!")
    return(-1)
  end if
  
  lAttraction = me.pAttractLoopList[me.pNextAttraction]
  
  GenLog(#debug, "AttractLoop::Cycle: Loading attraction" && string(lAttraction) && "...")
  _movie.puppetTransition(23, 2, 1, FALSE) 
  
  if (lAttraction.type = #bitmap) then
    lMember = member("attract_loop,image")
    
  else if (lAttraction.type = #movie) then
    lMember = member("attract_loop,mpeg")
  end if
  
  lMember.filename = me.pAttractLoopFolder & lAttraction.filename
  lTimeout = lAttraction.duration * 1000 -- ms = s * 1000
  
  if (SpriteExists("attraction")) then
    sprite("attraction").member = lMember
    sprite("attraction").volume = 120 -- 10%
  end if
  me._CenterSprite()
  
  me.StartCycling(lTimeout)
  
  me.pNextAttraction = me.pNextAttraction + 1
  if (me.pNextAttraction > me.pAttractLoopList.count) then
    me.pNextAttraction = 1
    me.pPlayCount = me.pPlayCount + 1
  end if
  
  me.pStarted = true
  
  GenLog(#debug, "AttractLoop::Cycle: Attraction loaded!")
end

-------------------------------------------------------------------------------
-- void StopCycling (me)
--
-- This method is called to stop the attract loop cycling. It kills the timeout.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on StopCycling (me)
  GenLog(#debug, "AttractLoop::StopCycling: Cycling stopped!")
  QI(#attractCycleTimeout).Stop()
  me.pStarted = false
end

on Watch (me)
  lSecMidnight = GetSecondsSinceMidnight()
  lLoopDuration = me.GetTotalDuration()
  
  if (me.GetStarted() = false) then
    -- Check if it's time to start the attract loop  
    lSecStart = integer(lSecMidnight mod lLoopDuration)
    lRemaining = me._GetNextStartTime(lSecStart)
    if (lRemaining <= 0 or lRemaining >= 120) then
      -- Start the attract loop
      lIndex = me._GetNextStartIndex(lSecStart)
      me.Cycle(lIndex)
      GenLog(#debug, "AttractLoop::Watch: Attract loop started!")
    else
      SetDebugInfo("Starting in"&&lRemaining&"...")
      GenLog(#debug, "AttractLoop::Watch: Starting in"&&lRemaining&&"second(s)...")
    end if
    
  else
    -- Attract loop is running, do we need to sync yet? 
    -- TODO Get x times loop duration from configuration
    lResetTime = 3 * lLoopDuration
    if (lSecMidnight mod lResetTime = 0) then
      -- Sync the attract loop
      me.Reset()
      GenLog(#debug, "AttractLoop::Watch: Attract loop reset!")
    else
      lRemaining = string(integer(lResetTime - (lSecMidnight mod lResetTime)))
      SetDebugInfo("Reset in"&&lRemaining&"...")
    end if
  end if
end

-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pAttractLoopList = []  
  me.pNextAttraction = 1
  me.pTotalDuration = 0
  me.pPlayCount = 0
  me.pStarted = false
  me.pAttractLoopFolder = ""
  me.pAttractLoopName = ""
end

-------------------------------------------------------------------------------
-- void _XMLCallback (me, theItemName, theItemValue)
--
-- Callback used by the XML parser to return data. This method is the brain of
-- the XML processing because it knows what it has to look for. It is specified
-- to the XML Parse method and is called everytime it encounters an XML tag.
-- If you need to retrieve information from a new tag, you just have to add
-- the symbol corresponding and use the appropriate path. You can easily
-- retrieve any data you want using GetItemValue from the XML parser. Then you
-- have to add the retieved information to the appropriate list.
--
-- Parameters:
--     me (object): this pointer.
--     theItemName (symbol): The current item name being parsed.
--     theItemValue (list): Contains the information for the current item.
--
-------------------------------------------------------------------------------
on _XMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #attraction:
      -- Get common information
      lName = me.GetItemValue(theItemValue, "name")
      lDescription = me.GetItemValue(theItemValue, "description")
      lDuration = integer(me.GetItemValue(theItemValue, "duration"))
      
      me.pTotalDuration = me.pTotalDuration + lDuration
      
      lCurrentAttraction = [:]
      lCurrentAttraction.addProp(#name, lName)
      lCurrentAttraction.addProp(#description, lDescription)
      lCurrentAttraction.addProp(#duration, lDuration)
      
      -- Get each possible types
      lFlash = me.GetItemValue(theItemValue, "flash")
      lImage = me.GetItemValue(theItemValue, "image")
      lMovie = me.GetItemValue(theItemValue, "movie")
      lChannel = me.GetItemValue(theItemValue, "channel")
      
      -- Get information according to the type
      if (lFlash <> void) then
        lCurrentAttraction.addProp(#type, #flash)
        lFlashPath = me.GetItemValue(lFlash, "child.value")
      else if (lImage <> void) then
        lCurrentAttraction.addProp(#type, #bitmap)
        lCurrentAttraction.addProp(#filename, lImage)
      else if (lMovie <> void) then
        lCurrentAttraction.addProp(#type, #movie)
        lCurrentAttraction.addProp(#filename, lMovie)
      else if (lChannel <> void) then
        -- TODO: Retrieve Channel information
      end if
      
      me.pDurationIndexList.add(me.pTotalDuration)
      
      me.pAttractLoopList.add(lCurrentAttraction)
  end case
end

on _CenterSprite (me)
  if (SpriteExists("attraction")) then
    sprite("attraction").rect = rect(0, 87, 1024, 663)
    sprite("attraction").visible = true
  end if
end

on _HideSprite (me)
  if (SpriteExists("attraction")) then
    sprite("attraction").visible = false
  end if
end

on _GetNextStartTime (me, theCurrTime)
  lCount = me.pDurationIndexList.count - 1
  repeat with i = 1 to lCount
    lLower = me.pDurationIndexList[i]
    lHigher = me.pDurationIndexList[i+1]
    if (lLower = theCurrTime) then
      return(0)
    else if (lLower < theCurrTime and lHigher > theCurrTime) then
      return(lHigher - theCurrTime)
    end if
  end repeat
  return(0)
end

on _GetNextStartIndex (me, theCurrTime)
  lCount = me.pDurationIndexList.count - 1
  repeat with i = 1 to lCount
    lLower = me.pDurationIndexList[i]
    lHigher = me.pDurationIndexList[i+1]
    if (lHigher = theCurrTime) then
      return(i+1)
    else if (lLower = theCurrTime) then
      return(i)
    end if
  end repeat
  return(0)
end