-------------------------------------------------------------------------------
-- Name: class Events
-- Author: Jonathan Primeau
--
-- Description:
--    This class manages events
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pEventList


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

on RegisterEvent (me, theEvent)
  GenLog(#debug, "Events::RegisterEvent: Registering event" && string(theEvent) & "...")
  lEventInfo = [:]
  lEventInfo.addProp(#enabled, 1)
  lEventInfo.addProp(#listeners, [])
  me.pEventList.addProp(theEvent, lEventInfo)
end

on EnableEvent (me, theEvent)
  if (me._EventExists(theEvent)) then
    GenLog(#debug, "Events::EnableEvent: Enabling event" && string(theEvent) & "...")
    me.pEventList.getProp(theEvent).setProp(#enabled, 1)
  end if
end

on DisableEvent (me, theEvent)
  if (me._EventExists(theEvent)) then
    GenLog(#debug, "Events::DisableEvent: Disabling event" && string(theEvent) & "...")
    me.pEventList.getProp(theEvent).setProp(#enabled, 0)
  end if
end

on IsEventEnabled (me, theEvent)
  return(me.pEventList.getProp(theEvent).getProp(#enabled))
end

on UnregisterEvent (me, theEvent)
  GenLog(#debug, "Events::UnregisterEvent: Unregistering event" && string(theEvent) & "...")
  me.pEventList.deleteProp(theEvent)
end

on RegisterListener (me, theEvent, theHandler, theObject)
  GenLog(#debug, "Events::RegisterListener: Registering listener" && string(theHandler) & "...")
  lListener = [:]
  lListener.addProp(#handler, theHandler)
  lListener.addProp(#object, theObject)
  me._AddListener(theEvent, lListener)
end

on UnregisterListener (me, theEvent, theHandler, theObject)
  GenLog(#debug, "Events::RegisterListener: Unregistering listener" && string(theHandler) & "...")
  lListener = [:]
  lListener.addProp(#handler, theHandler)
  lListener.addProp(#object, theObject)
  me._RemoveListener(theEvent, lListener)
end

on Trigger (me, theEvent, theParams)
  if (me._EventExists(theEvent)) then
    if ((me.IsEventEnabled(theEvent))) then
      lCount = me._GetListenerCount(theEvent)
      repeat with i = 1 to lCount
        lListener = me._GetListener(theEvent, i)
        if (lListener <> void) then
          --GenLog(#debug, "Events::Trigger: Triggering event" && string(theEvent) & "...")
          call(lListener.handler, lListener.object, theParams)
        end if
      end repeat
    end if
  end if
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pEventList = [:]
end

on _EventExists (me, theEvent)
  if (me.pEventList.getaProp(theEvent) <> void) then
    return(true)
  else
    return(false)
  end if
end

on _AddListener (me, theEvent, theListener)
  if (me._EventExists(theEvent)) then
    me.pEventList.getProp(theEvent).getProp(#listeners).add(theListener)
  end if
end

on _RemoveListener (me, theEvent, theListener)
  if (me._EventExists(theEvent)) then
    lCount = me._GetListenerCount(theEvent)
    repeat with i = 1 to lCount
      lListener = me._GetListener(theEvent, i)
      if (lListener.handler = theListener.handler and lListener.object = theListener.object) then
        me.pEventList.getProp(theEvent).getProp(#listeners).deleteAt(i)
        lCount = lCount - 1
        i = i - 1
      end if
    end repeat
  end if
end

on _GetListenerCount (me, theEvent)
  return(me.pEventList.getProp(theEvent).getProp(#listeners).count)
end

on _GetListener (me, theEvent, theIndex)
  if (theIndex > 0 and theIndex <= me._GetListenerCount(theEvent)) then
    return(me.pEventList.getProp(theEvent).getProp(#listeners).getAt(theIndex))
  end if
  return(void)
end