-------------------------------------------------------------------------------
-- Name: class ScriptManager
-- Author: Jonathan Primeau
--
-- Description:
--     This class is important. It is the first to be instantiate and it takes 
--     care of all the script objects (singleton). Every script used in the 
--     movie has to be instantiate from here. Also, this class provides a 
--     QueryInterface method that returns a instantiated object.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pScriptList           -- Script object list

-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

-------------------------------------------------------------------------------
-- me QueryInterface (me, theInstName)
--
-- Creates or returns a script object. It lookups it the script array and 
-- returns its instance. If there is no instance yet, it creates one by calling
-- _CreateInterface.
--
-- Parameters:
--     me (object): this pointer.
--     theInstName (symbol): Symbol corresponding to the script instance 
--                           in the list
--
-- Return:
--     Instance of the script.
--
-------------------------------------------------------------------------------
on QueryInterface (me, theInstName)
  if (me.pScriptList.getaProp(theInstName) <> void) then
    -- Get the object is in the script list
    lScript = me.pScriptList[theInstName]
    
    -- Create the object if it doesn't already exist
    if (objectP(lScript.object) = 0) then
      return(me._CreateInterface(theInstName, lScript.name))
    end if
    
    -- Object already exist
    return(lScript.object)
  else
    ShowMessage("Wrong script instance name:"&&string(theInstName))
  end if
end

-------------------------------------------------------------------------------
-- void CleanInterface (me, theInstName)
--
-- Deletes a script instance in the list.
--
-- Parameters:
--     me (object): this pointer.
--     theInstName (symbol): Symbol corresponding to the script instance 
--                           in the list
--
-------------------------------------------------------------------------------
on CleanInterface (me, theInstName)
  -- Get the object is in the script list
  lScript = me.pScriptList[theInstName]
  
  -- Create the object if it doesn't already exist
  if (objectP(lScript.object)) then
    lScript.object.Destroy()
    lScript.object = void
    me.pScriptList[theInstName].object = void
  end if
end

-------------------------------------------------------------------------------
-- void CleanAllInterface (me)
--
-- Deletes all script instances found in the list.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on CleanAllInterface (me)
  if (listP(me.pScriptList)) then
    lCount = me.pScriptList.count
    repeat with i = 1 to lCount
      lName = me.pScriptList.getPropAt(i)
      me.CleanInterface(lName)
    end repeat
  end if
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
-- NOTE: You have to add new script instances here!
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pScriptList = [:]
  
  -- Navigation Log script object
  me.pScriptList.addProp(#genLog, [#object: void, #name: "class Log"])
  
  -- Configuration script object
  me.pScriptList.addProp(#config, [#object: void, #name: "class Configuration"])
  
  -- Application Status script object
  me.pScriptList.addProp(#status, [#object: void, #name: "class Status"])
  
  -- Skin script object
  me.pScriptList.addProp(#skin, [#object: void, #name: "class Skin"])
  
  -- Banner script object
  me.pScriptList.addProp(#banner, [#object: void, #name: "class Banner"])
  
  -- Main Navigation script object
  me.pScriptList.addProp(#nav, [#object: void, #name: "class Navigation"])
  
  -- Global file utilities script object
  me.pScriptList.addProp(#file, [#object: void, #name: "class File"])
  
  -- Attract loop script object
  me.pScriptList.addProp(#attractLoop, [#object: void, #name: "class AttractLoop"])
  
  -- Server script object
  me.pScriptList.addProp(#server, [#object: void, #name: "class MUSClient"])
  
  -- Download script object
  me.pScriptList.addProp(#download, [#object: void, #name: "class Download"])
  
  -- Capabilities script object
  me.pScriptList.addProp(#cap, [#object: void, #name: "class Capabilities"])
  
  -- Internet list script object
  me.pScriptList.addProp(#internet, [#object: void, #name: "class InternetList"])
  
  -- Video list script object
  me.pScriptList.addProp(#video, [#object: void, #name: "class VideoList"])
  
  -- Web Browser script object
  me.pScriptList.addProp(#web, [#object: void, #name: "class WebBrowser"])
  
  -- Virtual Keyboard script object
  me.pScriptList.addProp(#vkb, [#object: void, #name: "class VirtualKeyboard"])
  
  -- Enterbet script object
  me.pScriptList.addProp(#enterbet, [#object: void, #name: "class Enterbet"])
  
  -- Services script object
  me.pScriptList.addProp(#services, [#object: void, #name: "class Services"])
  
  -- Handicapping script object
  me.pScriptList.addProp(#handicapping, [#object: void, #name: "class Handicapping"])
  
  -- Automatic Logout Timeout script object
  me.pScriptList.addProp(#autoLogoutTimeout, [#object: void, #name: "class AutoLogoutTimeout"])
  
  -- Application Timeout script object
  me.pScriptList.addProp(#appTimeout, [#object: void, #name: "class ApplicationTimeout"])
  
  -- AttractSync Timeout script object
  me.pScriptList.addProp(#attractSyncTimeout, [#object: void, #name: "class AttractSyncTimeout"])
  
  -- AttractCycle Timeout script object
  me.pScriptList.addProp(#attractCycleTimeout, [#object: void, #name: "class AttractCycleTimeout"])
  
  -- BannerCycle Timeout script object
  me.pScriptList.addProp(#bannerCycleTimeout, [#object: void, #name: "class BannerCycleTimeout"])
  
  -- Download Timeout script object
  me.pScriptList.addProp(#downloadTimeout, [#object: void, #name: "class DownloadTimeout"])
  
  -- WinTV script object
  me.pScriptList.addProp(#wintv, [#object: void, #name: "class WinTV"])
  
  -- Terminal Communicator script object
  me.pScriptList.addProp(#terminalCom, [#object: void, #name: "class TerminalCommunicator"])
  
  -- Operation Server Communicator script object
  me.pScriptList.addProp(#opsCom, [#object: void, #name: "class OPSCommunicator"])
  
  -- Application Events script object
  me.pScriptList.addProp(#events, [#object: void, #name: "class ApplicationEvents"])
  
  -- OPS Connection timeout script object
  me.pScriptList.addProp(#opsConnectionTimeout, [#object: void, #name: "class OPSConnectionTimeout"])
  
  -- Initialization script object
  me.pScriptList.addProp(#init, [#object: void, #name: "class Initialization"])
  
  -- Enterbet login script object
  me.pScriptList.addProp(#ebLogin, [#object: void, #name: "class EnterbetLogin"])
  
  -- Login script object
  me.pScriptList.addProp(#login, [#object: void, #name: "class Login"])
  
  -- Wait script object
  me.pScriptList.addProp(#waitTimeout, [#object: void, #name: "class WaitTimeout"])
  
  -- CRM script object
  me.pScriptList.addProp(#crm, [#object: void, #name: "class CRM"])
  
  -- Login timeout script object
  me.pScriptList.addProp(#loginTimeout, [#object: void, #name: "class LoginTimeout"])
  
  -- Logout timeout script object
  me.pScriptList.addProp(#logoutTimeout, [#object: void, #name: "class LogoutTimeout"])
  
  -- Offer timeout script object
  me.pScriptList.addProp(#offerTimeout, [#object: void, #name: "class OfferTimeout"])
  
  -- Terminal connection timeout script object
  me.pScriptList.addProp(#terminalConnectionTimeout, [#object: void, #name: "class TerminalConnectionTimeout"])
  
  -- Web Proxy script object
  me.pScriptList.addProp(#webProxy, [#object: void, #name: "class WebProxy"])
  
  -- Security Manager script object
  me.pScriptList.addProp(#security, [#object: void, #name: "class SecurityManager"])
  
  -- Activity Log script object
  me.pScriptList.addProp(#actLog, [#object: void, #name: "class ActivityLog"])
  
  -- Navigation timeout script object
  me.pScriptList.addProp(#navTimeout, [#object: void, #name: "class NavigationTimeout"])
  
  -- Clear error timeout script object
  me.pScriptList.addProp(#clearErrorTimeout, [#object: void, #name: "class ClearErrorTimeout"])
  
  -- Fatal error script object
  me.pScriptList.addProp(#fatalError, [#object: void, #name: "class FatalError"])
  
  -- Web denied script object
  me.pScriptList.addProp(#webDenied, [#object: void, #name: "class WebDenied"])
  
  -- WebLoad commented out for now until we find a solution BUG: 186
  -- Web loading script object
  --me.pScriptList.addProp(#webLoad, [#object: void, #name: "class WebLoad"])
  
  -- Patron script object
  me.pScriptList.addProp(#patron, [#object: void, #name: "class Patron"])
  
  -- Debug Info script object
  me.pScriptList.addProp(#debugInfo, [#object: void, #name: "class DebugInfo"])
  
  -- Initialize Timeout script object
  me.pScriptList.addProp(#initTimeout, [#object: void, #name: "class InitializeTimeout"])
end

-------------------------------------------------------------------------------
-- void _CreateInterface (me, theInstName, theScriptName)
--
-- This method creates a script object and sets it in the script list so it
-- can be accessed later.
--
-- Parameters:
--     me (object): this pointer.
--     theInstName (symbol): Symbol corresponding to the script instance 
--                           in the list
--     theScriptName (string): The script cast member name.
--
-- Return:
--     Instance of the created script.
--
-------------------------------------------------------------------------------
on _CreateInterface (me, theInstName, theScriptName)
  lObject = script(theScriptName).new()
  if (objectP(lObject)) then
    me.pScriptList[theInstName].object = lObject
    return(lObject)
  end if
  return(void)
end