-------------------------------------------------------------------------------
-- Name: class Status
-- Author: Jonathan Primeau
--
-- Description:
--     This class contains the current state of the movie For example, we want 
--     to keep the section currently displayed. Mostly made of Get / Set.
--
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pStatus               -- The current status
property pGroup                -- The current groups
property pLocation             -- The current physical location
property pKioskID              -- Kiosk ID sent by the server
property pKioskName            -- Kiosk name sent by the server


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class XMLParser").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

on GetCurrentBrowser (me)
  lCurrentSection = me.GetStatus().area
  if (lCurrentSection contains "WebXtra") then
    return("WebXtra")
  else if (lCurrentSection contains "MozillaXtra") then
    return("MozillaXtra")
  else if (lCurrentSection contains "WebViewerXtra") then
    return("WebViewerXtra")
  else 
    return("MozillaXtra")
  end if
end

on ResetStatus (me)
  me.pStatus = [:]
  me.pStatus.addProp(#login, "N/A")
  me.pStatus.addProp(#sequence, "Configuration")
  me.pStatus.addProp(#admission, "Enabled")
  me.pStatus.addProp(#state, "OK")
  me.pStatus.addProp(#area, "N/A")
end

on SetStatus (me, theAttribute, theValue)
  lValue = string(theValue)
  
  -- Validate area
  if (theAttribute = #area) then
    -- This should never happen!
    if (IsValidMarker(theValue) = false) then
      ShowMessage("Invalid section requested")
      return()
    end if
  end if
  
  if (theAttribute = #state) then
    if (theValue = "Error") then
      -- We will clear the state in a while
      QI(#clearErrorTimeout).Start()
    end if
  end if
  
  -- Check if the attribute is valid
  if (me.pStatus.getaProp(theAttribute) <> void) then
    me.pStatus.setProp(theAttribute, lValue)
  end if
end

on GetStatus (me)
  return(me.pStatus)
end

on SetGroup (me, theGroup)
  me.pGroup = theGroup
end

on GetGroup (me)
  return(me.pGroup)
end

on SetLocation (me, theLocation)
  me.pLocation = theLocation
end

on GetLocation (me)
  return(me.pLocation)
end

on SetKioskID (me, theKioskID)
  me.pKioskID = theKioskID
end

on GetKioskID (me)
  return(me.pKioskID)
end

on SetKioskName (me, theKioskName)
  me.pKioskName = theKioskName
end

on GetKioskName (me)
  return(me.pKioskName)
end

on GetRunMode (me)
  return(_system.environmentPropList.runMode)
end

on SetMaintenanceMessage (me)
  lName = me.GetKioskName()
  lGroup = me.GetGroup()
  lLocation = me.GetLocation()
  
  lMsg = "Name:" && lName & RETURN & RETURN
  lMsg = lMsg & "Group:" && lGroup & RETURN & RETURN
  lMsg = lMsg & "Location:" && lLocation
  member("maintenance,message").text = lMsg
end

on ServerSetStatus (me, theData)
  me.LoadXMLFromString(theData)
  lStatus = me.Parse(#_ServerSetStatusXMLCallback)
  if (lStatus <> 0) then
    GenLog(#error, "Status::ServerSetStatus: Invalid state")
    SvrLog("Status: Invalid state provided by the server")
    return(-1)
  end if
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.ResetStatus()
  me.pGroup = "Unknown"
  me.pLocation = "Unknown"
  me.pKioskID = "001"
  me.pKioskName = "kiosk_001"
end

on _ServerSetStatusXMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
      
    #state:
      lState = string(me.GetPathItemValue(theItemValue, "child.value"))
      me.SetStatus(#state, lState)
  end case
end