-------------------------------------------------------------------------------
-- Name: class LogoutTimeout
-- Author: Jonathan Primeau
--
-- Description:
--     Takes care of the logout timeout
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
------------------------------------------------------------------------------- 

property ancestor              -- Parent
property pSpriteName
property pTVMode
property pVolumeList
property pVolumeIndex
property pInitialized

property pCurrentChannel
property pDefaultChannel

property pIsMuted              -- True if muted by the user
property pIsUserMuted          -- True if muted by the application (not visible)
property pIsVisible            -- True if visible
property pIsFullScreen

property pEnterbetTVSize

-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
  
  QI(#events).UnregisterEvent(#ChannelChanged)
  
  -- Just so the TV doesn't bug us while developping ;)
  me.SetChannel(me.pDefaultChannel)
end

-------------------------------------------------------------------------------
-- void Prepare (me)
--
-- This method prepares WinTV to be used. Should be called only once when the
-- application initializes itself.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Prepare (me)
  GenLog(#debug, "WinTV::Prepare:")
  if (me.pInitialized = false) then
    
    -- From configuration
    me.SetChannel(me._GetConfigDefaultChannel())
    me.SetTuneCountry(me._GetConfigTuneCountry())
    me.SetToolbarPosition(me._GetConfigToolbarPosition())
    me.SetContrast(me._GetConfigContrast())
    me.SetBrightness(me._GetConfigBrightness())
    me.SetHue(me._GetConfigHue())
    me.SetSaturation(me._GetConfigSaturation())
    me.SetTuneSource(me._GetConfigTuneSource())
    me.SetContextMenu(me._GetConfigContextMenu())
    
    me.pInitialized = true
  end if
end

-------------------------------------------------------------------------------
-- void Show (me)
--
-- This method will set the WinTV visible according to the rectangle.
--
-- Parameters:
--     me (object): this pointer.
--     theRect (rect): Position of the WinTV sprite.
--
-------------------------------------------------------------------------------
on Show (me, theRect)
  GenLog(#debug, "WinTV::Show:")
  
  -- Set the appropriate rect
  if (theRect <> void) then
    -- Enterbet
    if (sprite(me.pSpriteName).rect <> theRect) then
      sprite(me.pSpriteName).rect = theRect
    end if
  else
    -- Video section
    lCurrentSection = QI(#status).GetStatus().area
    case (lCurrentSection) of
      "initialize":
        lInitRect = rect(2, 2, 1024, 750)
        if (sprite(me.pSpriteName).rect <> lInitRect) then
          sprite(me.pSpriteName).rect = rect(2, 2, 1024, 750)
        else
          -- We are already in the initialization section and
          -- the television is already displayed (BUG 199)
          return()
        end if
      "video":
        sprite(me.pSpriteName).rect = rect(1, 135, 342, 393)
      "video_live":
        sprite(me.pSpriteName).rect = rect(1, 135, 641, 616)
      "video_medium":
        sprite(me.pSpriteName).rect = rect(1, 135, 641, 616)
      "video_full":
        sprite(me.pSpriteName).rect = rect(2, 2, 1024, 768)
        me.pIsFullScreen = true
    end case
    if (lCurrentSection <> "initialize") then
      me.SetVolumeCustom()
    else
      -- Force the volume to 10% (BUG 199)
      QI(#wintv).SetVolume(10)
    end if
  end if
  
  -- We need controls
  me.SetEnable(true)
  
  -- Set mute back to user preference
  me.SetMute(me.pIsUserMuted)
  
  -- WinTV is now visible
  me.pIsVisible = true
end

-------------------------------------------------------------------------------
-- void Hide (me)
--
-- This method will set the WinTV invisible.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Hide (me)
  GenLog(#debug, "WinTV::Hide:")
  -- Free CPU resources
  me.SetEnable(false)
  
  -- Hiding the sprite
  sprite(me.pSpriteName).rect = rect(0, 2, 1, 3)
  
  -- It has to be muted when not visible
  me.SetMute(true)
  
  -- WinTV is NOT visible
  me.pIsVisible = false
end

-------------------------------------------------------------------------------
-- void SetEnable (me)
--
-- This method gives us access to the controls.
--
-- Parameters:
--     me (object): this pointer.
--     theState (boolean): True if enable
--
-------------------------------------------------------------------------------
on SetEnable (me, theState)
  GenLog(#debug, "WinTV::SetEnable:" && theState)
  sprite(me.pSpriteName).enabled = theState
end

-------------------------------------------------------------------------------
-- void GetEnable (me)
--
-- This returns the enabled state.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     True if enabled.
--
-------------------------------------------------------------------------------
on GetEnable (me)
  return(sprite(me.pSpriteName).enabled)
end

-------------------------------------------------------------------------------
-- void ChannelUp (me)
--
-- This method sets the channel up.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on ChannelUp (me)
  lOldChannel = me.GetChannel()
  sprite(me.pSpriteName).ChannelUp()
  lNewChannel = me.GetChannel()
  me.pCurrentChannel = lNewChannel
  GenLog(#debug, "WinTV::ChannelUp:" && me.pCurrentChannel)
  
  -- Trigger the event
  lParams = [:]
  lParams.addProp(#oldChannel, lOldChannel)
  lParams.addProp(#newChannel, lNewChannel)
  QI(#events).Trigger(#ChannelChanged, lParams)
end

-------------------------------------------------------------------------------
-- void ChannelDown (me)
--
-- This method sets the channel down.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on ChannelDown (me)
  lOldChannel = me.GetChannel()
  sprite(me.pSpriteName).ChannelDown()
  lNewChannel = me.GetChannel()
  me.pCurrentChannel = lNewChannel
  GenLog(#debug, "WinTV::ChannelDown:" && me.pCurrentChannel)
  
  -- Trigger the event
  lParams = [:]
  lParams.addProp(#oldChannel, lOldChannel)
  lParams.addProp(#newChannel, lNewChannel)
  QI(#events).Trigger(#ChannelChanged, lParams)
end

-------------------------------------------------------------------------------
-- void SetChannel (me)
--
-- This method sets the channel.
--
-- Parameters:
--     me (object): this pointer.
--     theChannel (integer): The channel.
--
-------------------------------------------------------------------------------
on SetChannel (me, theChannel)
  lOldChannel = me.GetChannel()
  GenLog(#debug, "WinTV::SetChannel:" && theChannel)
  sprite(me.pSpriteName).channel = theChannel
  lNewChannel = me.GetChannel()
  me.pCurrentChannel = lNewChannel
  GenLog(#debug, "WinTV::SetChannel:" && me.pCurrentChannel)
  
  -- Trigger the event
  lParams = [:]
  lParams.addProp(#oldChannel, lOldChannel)
  lParams.addProp(#newChannel, lNewChannel)
  QI(#events).Trigger(#ChannelChanged, lParams)
end

-------------------------------------------------------------------------------
-- void GetChannel (me)
--
-- This returns the channel.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The current channel.
--
-------------------------------------------------------------------------------
on GetChannel (me)
  return(sprite(me.pSpriteName).channel)
end

-------------------------------------------------------------------------------
-- void SetContrast (me)
--
-- This method sets the contrast.
--
-- Parameters:
--     me (object): this pointer.
--     theConstrast (integer): The contrast.
--
-------------------------------------------------------------------------------
on SetContrast (me, theConstrast)
  GenLog(#debug, "WinTV::SetContrast:" && theConstrast)
  sprite(me.pSpriteName).contrast = theConstrast
end

-------------------------------------------------------------------------------
-- void GetConstrast (me)
--
-- This returns the contrast.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The current constrast.
--
-------------------------------------------------------------------------------
on GetConstrast (me)
  return(sprite(me.pSpriteName).contrast)
end

-------------------------------------------------------------------------------
-- void SetBrightness (me)
--
-- This method sets the brightness.
--
-- Parameters:
--     me (object): this pointer.
--     theBrightness (integer): The brightness.
--
-------------------------------------------------------------------------------
on SetBrightness (me, theBrightness)
  GenLog(#debug, "WinTV::SetBrightness:" && theBrightness)
  sprite(me.pSpriteName).brightness = theBrightness
end

-------------------------------------------------------------------------------
-- void GetBrightness (me)
--
-- This returns the brightness.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The current brightness.
--
-------------------------------------------------------------------------------
on GetBrightness (me)
  return(sprite(me.pSpriteName).brightness)
end

-------------------------------------------------------------------------------
-- void SetHue (me)
--
-- This method sets the hue.
--
-- Parameters:
--     me (object): this pointer.
--     theHue (integer): The hue.
--
-------------------------------------------------------------------------------
on SetHue (me, theHue)
  GenLog(#debug, "WinTV::SetHue:" && theHue)
  sprite(me.pSpriteName).hue = theHue
end

-------------------------------------------------------------------------------
-- void GetHue (me)
--
-- This returns the hue.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The current hue.
--
-------------------------------------------------------------------------------
on GetHue (me)
  return(sprite(me.pSpriteName).hue)
end

-------------------------------------------------------------------------------
-- void SetSaturation (me)
--
-- This method sets the saturation.
--
-- Parameters:
--     me (object): this pointer.
--     theSaturation (integer): The saturation.
--
-------------------------------------------------------------------------------
on SetSaturation (me, theSaturation)
  GenLog(#debug, "WinTV::SetSaturation:" && theSaturation)
  sprite(me.pSpriteName).saturation = theSaturation
end

-------------------------------------------------------------------------------
-- void GetSaturation (me)
--
-- This returns the saturation.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The current saturation.
--
-------------------------------------------------------------------------------
on GetSaturation (me)
  return(sprite(me.pSpriteName).saturation)
end

-------------------------------------------------------------------------------
-- void SetMute (me)
--
-- This method sets the mute state.
--
-- Parameters:
--     me (object): this pointer.
--     theState (boolean): True if muted.
--
-------------------------------------------------------------------------------
on SetMute (me, theState)
  GenLog(#debug, "WinTV::SetMute:" && theState)
  sprite(me.pSpriteName).mute = theState
  me.pIsMuted = theState
end

-------------------------------------------------------------------------------
-- void GetMute (me)
--
-- This returns the mute state.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The current mute state.
--
-------------------------------------------------------------------------------
on GetMute (me)
  return(sprite(me.pSpriteName).mute)
end

-------------------------------------------------------------------------------
-- void SetUserMute (me)
--
-- This method sets the user mute state.
--
-- Parameters:
--     me (object): this pointer.
--     theState (boolean): True if user muted.
--
-------------------------------------------------------------------------------
on SetUserMute (me, theState)
  -- User can set the mute state only when visible
  if (me.pIsVisible = true) then
    GenLog(#debug, "WinTV::SetUserMute:" && theState)
    sprite(me.pSpriteName).mute = theState
    me.pIsUserMuted = theState
  else
    GenLog(#debug, "WinTV::SetUserMute: User tried to set mute state while not visible")
  end if
end

-------------------------------------------------------------------------------
-- void GetUserMute (me)
--
-- This returns the user mute state.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The current user mute state.
--
-------------------------------------------------------------------------------
on GetUserMute (me)
  return(me.pIsUserMuted)
end

-------------------------------------------------------------------------------
-- void SetVolume (me)
--
-- This method sets the volume.
--
-- Parameters:
--     me (object): this pointer.
--     theVolume (integer): The volume.
--
-------------------------------------------------------------------------------
on SetVolume (me, theVolume)
  GenLog(#debug, "WinTV::SetVolume:" && theVolume)
  sprite(me.pSpriteName).volume = theVolume
end

-------------------------------------------------------------------------------
-- void GetVolume (me)
--
-- This returns the volume.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The current volume.
--
-------------------------------------------------------------------------------
on GetVolume (me)
  return(sprite(me.pSpriteName).volume)
end

-------------------------------------------------------------------------------
-- void SetToolbarPosition (me)
--
-- This method sets the toolbar position.
--
-- Parameters:
--     me (object): this pointer.
--     thePos (integer): The Position (4=invisible).
--
-------------------------------------------------------------------------------
on SetToolbarPosition (me, thePos)
  GenLog(#debug, "WinTV::SetToolbarPosition:" && thePos)
  sprite(me.pSpriteName).toolBarPosition = thePos
end

-------------------------------------------------------------------------------
-- void SetTuneSource (me)
--
-- This method sets the tune source.
--
-- Parameters:
--     me (object): this pointer.
--     theTuneSource (integer): The tune source (0=antenna, 1=cable).
--
-------------------------------------------------------------------------------
on SetTuneSource (me, theTuneSource)
  GenLog(#debug, "WinTV::SetTuneSource:" && theTuneSource)
  sprite(me.pSpriteName).tuneSource = theTuneSource 
end

-------------------------------------------------------------------------------
-- void GetTuneSource (me)
--
-- This returns the tune source.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The current tune source.
--
-------------------------------------------------------------------------------
on GetTuneSource (me)
  return(sprite(me.pSpriteName).tuneSource)
end

-------------------------------------------------------------------------------
-- void SetTuneCountry (me)
--
-- This method sets the tune country.
--
-- Parameters:
--     me (object): this pointer.
--     theTuneCountry (integer): The tune country (0=USA, 1=Canada).
--
-------------------------------------------------------------------------------
on SetTuneCountry (me, theTuneCountry)
  sprite(me.pSpriteName).tuneCountry = theTuneCountry
end

-------------------------------------------------------------------------------
-- void GetCountry (me)
--
-- This returns the country.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The current country.
--
-------------------------------------------------------------------------------
on GetCountry (me)
  return(sprite(me.pSpriteName).tuneCountry)
end

-------------------------------------------------------------------------------
-- void VolumeUp (me)
--
-- This method sets the volume up.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on VolumeUp (me)
  sprite(me.pSpriteName).VolumeUp()
  GenLog(#debug, "WinTV::VolumeUp:")
end

-------------------------------------------------------------------------------
-- void VolumeDown (me)
--
-- This method sets the volume down.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on VolumeDown (me)
  sprite(me.pSpriteName).VolumeDown()
  GenLog(#debug, "WinTV::VolumeDown:")
end

-------------------------------------------------------------------------------
-- void VolumeUpCustom (me)
--
-- This method sets the volume up using the predetermined values.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on VolumeUpCustom (me)
  lCount = me.pVolumeList.count
  if (me.pVolumeIndex < lCount) then
    me.pVolumeIndex = me.pVolumeIndex + 1
    me.SetVolumeCustom()
  end if
end

-------------------------------------------------------------------------------
-- void VolumeDownCustom (me)
--
-- This method sets the volume down using the predetermined values.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on VolumeDownCustom (me)
  if (me.pVolumeIndex > 1) then
    me.pVolumeIndex = me.pVolumeIndex - 1
    me.SetVolumeCustom()
  end if
end

-------------------------------------------------------------------------------
-- void SetVolumeCustom (me)
--
-- This method sets the volume using the predetermined values.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on SetVolumeCustom (me)
  sprite(me.pSpriteName).volume = me.pVolumeList[me.pVolumeIndex]
  
  -- Set the appropriate volume indicator
  lSection = QI(#status).GetStatus().area
  if (lSection = "video" or lSection = "video_medium" or lSection = "video_live") then
    sprite("volume").member = lSection & ",volume_" & string(me.pVolumeIndex - 1)
  end if
  
  GenLog(#debug, "WinTV::SetVolumeCustom:" && me.pVolumeList[me.pVolumeIndex])
end

on ResetVolumeCustom (me)
  me.pVolumeIndex = 2
end

-------------------------------------------------------------------------------
-- void SetTVMode (me)
--
-- This method sets the TV mode (fullscreen).
--
-- Parameters:
--     me (object): this pointer.
--     theState (integer): The TV mode (0=Normal, 1=Fullscreen).
--
-------------------------------------------------------------------------------
on SetTVMode (me, theState)
  sprite(me.pSpriteName).tvMode = theState
  me.pTVMode = theState
  GenLog(#debug, "WinTV::SetTVMode:" && theState)
end

-------------------------------------------------------------------------------
-- void GetTVMode (me)
--
-- This returns the TV mode.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The current TV mode.
--
-------------------------------------------------------------------------------
on GetTVMode (me)
  return(sprite(me.pSpriteName).tvMode)
end

-------------------------------------------------------------------------------
-- void SetContextMenu (me)
--
-- This method sets context menu state.
--
-- Parameters:
--     me (object): this pointer.
--     theState (integer): The context menu state.
--
-------------------------------------------------------------------------------
on SetContextMenu (me, theState)
  sprite(me.pSpriteName).contextMenuEnabled = theState
  GenLog(#debug, "WinTV::SetContextMenu:" && theState)
end

-------------------------------------------------------------------------------
-- void GetContextMenu (me)
--
-- This returns the context menu.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The current context menu.
--
-------------------------------------------------------------------------------
on GetContextMenu (me)
  return(sprite(me.pSpriteName).contextMenuEnabled)
end

on CheckFullScreen (me)
  lSection = QI(#status).GetStatus().area
  if (lSection = "wagering") then
    if (me.pIsFullScreen = true) then
      if (me.pEnterbetTVSize = "small") then
        me.Show(rect(200, 210, 484, 420))
      else
        me.Show(rect(550, 170, 970, 490))
      end if 
      me.pIsFullScreen = false
    else
      sprite(me.pSpriteName).rect = rect(1, 0, 1024, 768)
      me.pIsFullScreen = true
    end if
  else if (lSection = "video_full") then
    if (me.pIsFullScreen = true) then
      QI(#nav).GoBack(true)
      me.pIsFullScreen = false
    end if
  end if
end


-------------------------------------------------------------------------------
-- E N T E R B E T  E N T R Y  P O I N T S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void EnterbetSetTVSize (me)
--
-- Called by enterbet to set the TV size.
--
-- Parameters:
--     me (object): this pointer.
--     theSize (string): "small" or "large"
--
-------------------------------------------------------------------------------
on EnterbetSetTVSize (me, theSize)
  GenLog(#debug, "WinTV::EnterbetSetTVSize:" && theSize)
  me.pEnterbetTVSize = theSize
end

-------------------------------------------------------------------------------
-- void EnterbetSetTVVisible (me)
--
-- Called by enterbet to set the TV visibility.
--
-- Parameters:
--     me (object): this pointer.
--     theState (string): "true" or "false"
--
-------------------------------------------------------------------------------
on EnterbetSetTVVisible (me, theState)
  GenLog(#debug, "WinTV::EnterbetSetTVVisible:" && theState)
  
  if (theState <> me.pIsVisible) then
    lState = value(theState)
    
    if (lState = true) then
      if (me.pEnterbetTVSize = "small") then
        me.Show(rect(200, 210, 484, 420))
      else
        me.Show(rect(550, 170, 970, 490))
      end if  
    else
      me.Hide()
    end if
  end if
end

-------------------------------------------------------------------------------
-- void EnterbetSetTVChannel (me)
--
-- Called by enterbet to set the TV channel.
--
-- Parameters:
--     me (object): this pointer.
--     theChannel (string): The channel
--
-------------------------------------------------------------------------------
on EnterbetSetTVChannel (me, theChannel)
  GenLog(#debug, "WinTV::EnterbetSetTVChannel:" && theChannel)
  QI(#winTV).SetChannel(integer(theChannel))
end

-------------------------------------------------------------------------------
-- void EnterbetSetTVVolume (me)
--
-- Called by enterbet to set the TV volume.
--
-- Parameters:
--     me (object): this pointer.
--     theChannel (string): The volume
--
-------------------------------------------------------------------------------
on EnterbetSetTVVolume (me, theVolume)
  GenLog(#debug, "WinTV::EnterbetSetTVVolume:" && theVolume)
  QI(#winTV).SetVolume(integer(theVolume))
end

-------------------------------------------------------------------------------
-- void EnterbetSetTVMute (me)
--
-- Called by enterbet to set the TV mute.
--
-- Parameters:
--     me (object): this pointer.
--     theState (string): "true" or "false"
--
-------------------------------------------------------------------------------
on EnterbetSetTVMute (me, theState)
  GenLog(#debug, "WinTV::EnterbetSetTVMute:" && theState)
  QI(#winTV).SetUserMute(value(theState))
end

on EnterbetSetFullScreen (me, theState)
  GenLog(#debug, "WinTV::EnterbetSetFullScreen:" && theState)
  -- TODO
  --ShowMessage("WinTV::EnterbetSetFullScreen: not implemented!")
end

on EnterbetToggleFullScreen (me)
  GenLog(#debug, "WinTV::EnterbetToggleFullScreen:")
  -- TODO
  --ShowMessage("WinTV::EnterbetToggleFullScreen: not implemented!")
end

on EnterbetCaptureImage (me, theIndex)
  GenLog(#debug, "WinTV::EnterbetCaptureImage:" && theIndex)
  
  lFileName = "c:\snapshot" & theIndex & ".jpg"
  sprite(me.pSpriteName).saveToDiskEx(lFileName, 2, 32, 1, 90)
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pVolumeList = [0,20,40,50,60,80,100]
  me.ResetVolumeCustom()
  
  me.pDefaultChannel = 3
  
  me.pSpriteName = "WinTV"
  me.pCurrentChannel = 0
  me.pTVMode = false
  me.pInitialized = false
  
  me.pIsMuted = true
  me.pIsUserMuted = false
  me.pIsVisible = false
  me.pIsFullScreen = false
  
  me.pEnterbetTVSize = "small"
  
  QI(#events).RegisterEvent(#ChannelChanged)
  QI(#video).RegisterListeners()
end

on _GetConfigDefaultChannel (me)
  lInfo = QI(#config).GetVideoSettings()
  if (lInfo.getaProp(#defaultChannel) <> void) then
    return(integer(lInfo.defaultChannel))
  else
    return(me.pDefaultChannel)
  end if
end

-------------------------------------------------------------------------------
-- void _GetConfigContrast (me)
--
-- Retrieves contrast from configuration.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The configuration contrast.
--
-------------------------------------------------------------------------------
on _GetConfigContrast (me)
  lInfo = QI(#config).GetVideoSettings()
  if (lInfo.getaProp(#contrast) <> void) then
    return(integer(lInfo.contrast))
  else
    return(128)
  end if
end

-------------------------------------------------------------------------------
-- void _GetConfigBrightness (me)
--
-- Retrieves brightness from configuration.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The configuration brightness.
--
-------------------------------------------------------------------------------
on _GetConfigBrightness (me)
  lInfo = QI(#config).GetVideoSettings()
  if (lInfo.getaProp(#brightness) <> void) then
    return(integer(lInfo.brightness))
  else
    return(128)
  end if
end

-------------------------------------------------------------------------------
-- void _GetConfigHue (me)
--
-- Retrieves hue from configuration.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The configuration hue.
--
-------------------------------------------------------------------------------
on _GetConfigHue (me)
  lInfo = QI(#config).GetVideoSettings()
  if (lInfo.getaProp(#hue) <> void) then
    return(integer(lInfo.hue))
  else
    return(128)
  end if
end

-------------------------------------------------------------------------------
-- void _GetConfigSaturation (me)
--
-- Retrieves saturation from configuration.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The configuration saturation.
--
-------------------------------------------------------------------------------
on _GetConfigSaturation (me)
  lInfo = QI(#config).GetVideoSettings()
  if (lInfo.getaProp(#saturation) <> void) then
    return(integer(lInfo.saturation))
  else
    return(128)
  end if
end

-------------------------------------------------------------------------------
-- void _GetConfigTuneSource (me)
--
-- Retrieves tune source from configuration.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The configuration tune source.
--
-------------------------------------------------------------------------------
on _GetConfigTuneSource (me)
  lInfo = QI(#config).GetVideoSettings()
  if (lInfo.getaProp(#tuneSource) <> void) then
    return(integer(lInfo.tuneSource))
  else
    return(0)
  end if
end

-------------------------------------------------------------------------------
-- void _GetConfigContextMenu (me)
--
-- Retrieves context menu state from configuration.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The configuration context menu state.
--
-------------------------------------------------------------------------------
on _GetConfigContextMenu (me)
  lInfo = QI(#config).GetVideoSettings()
  if (lInfo.getaProp(#contextMenu) <> void) then
    return(integer(lInfo.contextMenu))
  else
    return(1)
  end if
end

-------------------------------------------------------------------------------
-- void _GetConfigToolbarPosition (me)
--
-- Retrieves toolbar position from configuration.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The configuration toolbar position.
--
-------------------------------------------------------------------------------
on _GetConfigToolbarPosition (me)
  lInfo = QI(#config).GetVideoSettings()
  if (lInfo.getaProp(#toolbarPosition) <> void) then
    return(integer(lInfo.toolbarPosition))
  else
    return(0)
  end if
end

-------------------------------------------------------------------------------
-- void _GetConfigTuneCountry (me)
--clear
-- Retrieves tune country from configuration.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The configuration tune country.
--
-------------------------------------------------------------------------------
on _GetConfigTuneCountry (me)
  lInfo = QI(#config).GetVideoSettings()
  if (lInfo.getaProp(#tuneCountry) <> void) then
    return(integer(lInfo.tuneCountry))
  else
    return(0)
  end if
end